#include <qthing.hpp>
#include <util.hpp>

qthing::RGBW8 hsvToRgb(uint16_t h, uint16_t s, uint16_t v) {
  qthing::RGBW8 color;

  if (s > 255) s = 255;
  if (v > 255) v = 255;

  auto i = (h * 6) / 256;
  auto f = (h * 6) % 256;
  auto p = (v * (255 - s)) / 256;
  auto q = (v * (255 - (f * s / 256))) / 256;
  auto t = (v * (255 - (((255 - f) * s) / 256))) / 256;

  switch(i % 6) {
    case 0: color.r = v; color.g = t; color.b = p; break;
    case 1: color.r = q; color.g = v; color.b = p; break;
    case 2: color.r = p; color.g = v; color.b = t; break;
    case 3: color.r = p; color.g = q; color.b = v; break;
    case 4: color.r = t; color.g = p; color.b = v; break;
    case 5: color.r = v; color.g = p; color.b = q; break;
  }

  return color;
}

qthing::RGBW8 color_handler(uint8_t strip_id, uint16_t address, qthing::RGBW8 last_color) {
  int lowerBound = 2;
  int upperBound = 250;
  uint h = ( (millis()/50)%256 + (-address*10)%256 )%256;
  uint s = 196;
  uint v = lowerBound+labs(-(upperBound-lowerBound)+(millis()/70)%(2*upperBound));
  return hsvToRgb(h, s, v);
}

void device_main() {
  qthing::add_ledstrip(GPIO_NUM_4, 50, qthing::LED_WS2812B_V2);
  qthing::add_ledstrip(GPIO_NUM_5, 50, qthing::LED_WS2812B_V2);
  qthing::set_color_handler(color_handler);
  qthing::enable_leds(60);
}
